#!/usr/bin/python
# I_waveform_envelope_spectrum_distance_clusters.py
# Minimal code for plotting waveform and envelope. D. Gibbon, 2021-07-06

import sys, re, glob											# import specialised modules
import numpy as np
import matplotlib.pyplot as plt
import scipy.io.wavfile as wave
from scipy.signal import butter, lfilter
import scipy.spatial.distance as dist
import scipy.cluster.hierarchy as hy
from graphviz import Graph

figwidth = 6.5
figheight = 4
boxwidth = 0.6
boxheight = 0.83
halign = 0.02
valign = 0.14
# Change this if necessary (options: left, right, top, bottom)
orientation = "left"

dendrolevels = 20
spectrummax = 3
formantlimit = 0.8

distancemetrics = [ 'canberra', 'chebyshev', 'cityblock', 'correlation', 'cosine', 'euclidean' ]
methodlist_euclid = [ 'single', 'complete', 'average', 'weighted', 'centroid', 'median',  'ward' ]
methodlist_other = [ 'single', 'complete', 'average', 'weighted' ]

# Collect spectrum vectors

if len(sys.argv) > 1:
	wavfiledirectory = sys.argv[1]						# get input filename from command line 
	datasetname = sys.argv[2]
else:
	wavfiledirectory = "DATA/DATA_RT/"
	datasetname = "RFA_RT"

wavfilelist = sorted(glob.glob(wavfiledirectory+"*.wav"))

namelist = []
rawvaluelist = []
for wavfilename in wavfilelist:
	wavfilebase = re.sub(".*/", "", wavfilename)
	wavfilebase = re.sub("-mono-16k.wav","",wavfilebase)
	figurefilebase = "DGRAMS/I_%s"%wavfilebase

	fs, signal = wave.read(wavfilename)						# read sampling frequency and signal

	signallength = len(signal)								# define signal length in bytes
	signalseconds = int(signallength / fs)					# define signal length in seconds
	signal = signal / max(abs(signal))						# normalise signal -1 ... 0 ... 1

	b, a = butter(5, 10 / (0.5 * fs), btype="low")			# define Butterworth filter
	envelope = lfilter(b, a, abs(signal))					# apply filter to create lf envelope
	envelope = envelope / max(envelope)						# normalise envelope 0 ... 1

	specmags = np.abs(np.fft.rfft(envelope))
	specmaglen = len(specmags)
	lfspecmaglen = int(round(spectrummax * specmaglen / (fs / 2)))
	lfspecmags = specmags[1:lfspecmaglen]
	lfspecmags = lfspecmags / max(lfspecmags)
	lfspecmags = [ 0 if m <= formantlimit else m for m in lfspecmags ]	# focus on formants
	namelist += [ wavfilebase ]
	rawvaluelist += [ lfspecmags ]

# Equalise lengths of data
newsize = np.max( [ len(val) for val in rawvaluelist ] )
valuelist = []
for val in rawvaluelist:
	size = len(val)
	xloc = np.arange(size)
	new_xloc = np.linspace(0, size, newsize)
	new_data = np.interp(new_xloc, xloc, val)
	valuelist += [ new_data ]

valuelist = np.array(valuelist)

# Create distance networks with different metrics using GraphViz
for distancemetric in distancemetrics:

	print("\nProcessing %s with %s"%(wavfilebase, distancemetric))
	distances = dist.pdist(valuelist, metric=distancemetric)

	if distancemetric == "euclidean":
		clustermethods = methodlist_euclid
	else:
		clustermethods = methodlist_other

	for clustermethod in clustermethods:
			print("Distance metric:", distancemetric, "Clustering method:", clustermethod)
			fig = plt.figure(figsize=(figwidth, figheight))
			ax1 = fig.add_axes([halign, valign, boxwidth, boxheight])
			ax1.set_xlabel("%s%s-%s"%(figurefilebase,distancemetric,clustermethod), fontsize=8)
			Y1 = hy.linkage(distances, method=clustermethod)
			hy.dendrogram(Y1,
				p = dendrolevels, truncate_mode = "level",
				orientation=orientation,
#				cutoff = 0.3*np.max(Y1[:,2])
				above_threshold_color='black', color_threshold=0,
				count_sort="False",
				distance_sort=False, labels=namelist, leaf_font_size=10)
			figurefilename = figurefilebase + "-%s-%s-dendro.png"%(distancemetric,clustermethod)
			plt.savefig(figurefilename)

			plt.show()

			plt.close(fig)	# Close each graph in loop after saving and displaying

